xquery version "3.0" encoding "UTF-8";

(: Handler queries called by controller :)

module namespace g = "http://www.brueggem.de/demos/guess";
import module namespace gm = "http://www.brueggem.de/demos/guess/model" at "model.xqm";

declare function g:init() as node() {
    g:showInitialScreen(gm:getDefaultRange())
};

declare function g:quit() as node() {
    g:showQuitScreen()
};

declare function g:handleStart($range as xs:positiveInteger) as node() {
    let $temp := gm:startGame($range)
    return g:showFirstGameScreen(gm:getGuessesLeft())
};

declare function g:handleGuess($guess as xs:nonNegativeInteger) as node() {
    let $result := gm:handleGuess($guess)
    let $guessesLeft := gm:getGuessesLeft()
    let $resultInWords := if ($result = $gm:high) then "high" else if ($result = $gm:low) then "low" else "correct"
    return if ($result = $gm:correct) then g:showFinalScreen(xs:boolean("true"), gm:getSecretNumber())
        else if ($guessesLeft = 0) then g:showFinalScreen(xs:boolean("false"), gm:getSecretNumber())
        else g:showGameScreen($resultInWords, $guessesLeft)
};

declare function g:showInitialScreen($range as xs:positiveInteger) as node() {
    <html>
        <body>
            <h1>Initial Screen</h1>
            <p>Enter range and start game!</p>
            <form
                autocomplete="off"
				method="POST"
				action="play.xq">
				<input
				    type="hidden"
				    name="cmd"
				    value="start"/>
				<input
					type="text"
					name="arg"
					value="{$range}"
					size="5"/>
				<input
					type="submit"
					value="start"/>
			</form>
        </body>
    </html>
};

declare function g:showFirstGameScreen($guessesLeft as xs:positiveInteger) as node() {
    <html>
        <body>
            <h1>Game Screen</h1>
            <p>Guesses left: {$guessesLeft}</p>
            <p>Enter guess!</p>
            <form
                autocomplete="off"
				method="POST"
				action="play.xq">
				<input
				    type="hidden"
				    name="cmd"
				    value="guess"/>
				<input
					type="text"
					name="arg"
					size="5"/>
				<input
					type="submit"
					value="guess"/>
			</form>
        </body>
    </html>
};

declare function g:showGameScreen($resultInWords as xs:string,$guessesLeft as xs:positiveInteger) as node() {
    <html>
        <body>
            <h1>Game Screen</h1>
            <p>Previous guess was {$resultInWords}.</p>
            <p>Guesses left: {$guessesLeft}</p>
            <p>Enter guess!</p>
            <form
                autocomplete="off"
				method="POST"
				action="play.xq">
				<input
				    type="hidden"
				    name="cmd"
				    value="guess"/>
				<input
					type="text"
					name="arg"
					size="5"/>
				<input
					type="submit"
					value="guess"/>
			</form>
        </body>
    </html>
};

declare function g:showFinalScreen($isWon as xs:boolean, $secretNumber as xs:positiveInteger) as node() {
    <html>
        <body>
            <h1>Final Screen</h1>
            <p>{if ($isWon) then "You win" else "You loose"}</p>
            <p>Secret number: {$secretNumber}</p>
            <form
                autocomplete="off"
				method="POST"
				action="play.xq">
				<button
				    type="submit"
				    name="cmd"
				    value="init">again</button>
				<button
				    type="submit"
				    name="cmd"
				    value="quit">quit</button>
			</form>
        </body>
    </html>
};

declare function g:showQuitScreen() as node() {
    <html>
        <body>
            <h1>Quit Screen</h1>
            <p>Good bye</p>
        </body>
    </html>
};
