xquery version "3.0" encoding "UTF-8";

module namespace g = "http://www.brueggem.de/demos/guess/model";
declare variable $g:attributes := doc("attributes.xml");
declare variable $g:high as xs:integer := 1;
declare variable $g:low as xs:integer := -1;
declare variable $g:correct as xs:integer := 0;

(: This module stores its attributes in $g:attributes.
 : Note that all attributes must be initialized with non-empty
 : values for replace updates to work.
 : :)

declare function g:setDefaultRange($range as xs:positiveInteger) as empty-sequence() {
    update replace $g:attributes//defaultRange/text() with $range
};

declare function g:startGame($range as xs:positiveInteger) as empty-sequence() {
    (: util:random is an eXist extension function that returns a random number in [0,theArgument) :)
    let $secretNumber := util:random($range+1)
    (: this is overly generous, should be ceil(log($range)) :)
    let $guessesLeft := $range
    return
        (update replace $g:attributes//secretNumber/text() with $secretNumber,
        update replace $g:attributes//guessesLeft/text() with $guessesLeft)
};

declare function g:handleGuess($guess as xs:nonNegativeInteger) as xs:integer {
    (: only call if there are guesses left and game is not yet over :)
    let $secretNumber := g:getSecretNumber()
    let $guessesLeft := g:getGuessesLeft() - 1
    let $result := if ($guess < $secretNumber) then $g:low else if
        ($guess = $secretNumber) then $g:correct else $g:high
    let $temp := update replace $g:attributes//guessesLeft/text() with $guessesLeft
    return $result
};

declare function g:getSecretNumber() as xs:nonNegativeInteger {
    (: will only be called when game is over :)
    xs:nonNegativeInteger($g:attributes//secretNumber)
};

declare function g:getGuessesLeft() as xs:nonNegativeInteger {
    xs:nonNegativeInteger($g:attributes//guessesLeft)
};

declare function g:getDefaultRange() as xs:positiveInteger {
    xs:positiveInteger($g:attributes//defaultRange)
};

